use config::{Config, ConfigError, File};
use serde::Deserialize;
use std::path::PathBuf;

/// World types supported
#[derive(Debug, Deserialize, Clone)]
pub enum WorldType {
    /// The End (DIM1)
    END,
    /// Nether (DIM-1)
    NETHER,
    /// Overworld
    OVERWORLD,
}

impl From<String> for WorldType {
    /// Convert config strings to WorldType
    fn from(string: String) -> Self {
        match string.as_str() {
            "END" => WorldType::END,
            "NETHER" => WorldType::NETHER,
            _ => WorldType::OVERWORLD,
        }
    }
}

/// Config for individual WorldConfig
#[derive(Debug, Deserialize, Clone)]
pub struct WorldConfig {
    pub world_name: String,
    pub save_radius: u64,
    pub world_type: Option<WorldType>,
}

/// Config for doing backups
#[derive(Debug, Deserialize, Clone)]
pub struct BackupConfig {
    pub minecraft_dir: PathBuf,
    pub output_dir: PathBuf,
    pub backups_to_keep: u64,
    pub discord_webhook: Option<String>,
}

/// Configs
#[derive(Debug, Deserialize, Clone)]
pub struct AlbatrossConfig {
    pub backup: BackupConfig,
    pub world_config: Option<Vec<WorldConfig>>,
}

impl AlbatrossConfig {
    /// Create new backup from file
    pub fn new(config_path: &str) -> Result<Self, ConfigError> {
        let mut cfg = Config::new();
        cfg.merge(File::with_name(config_path))?;

        cfg.try_into()
    }
}
